"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import re

# Using default empty strings for query, lang, ext to avoid &ext=None etc.
meta = {
	'name': 'Zlibrary Search',
	'author': 'Vikas Kundu',
	'version': '0.1',
	'description': 'Search books and articles on Zlibrary . ',
	'sources': ('Zlibrary',),
	'options': (
		('query', '', True, 'Query string to search for', '-q', 'store', str),
		('exact', False, False, 'Search for exact query(True or False, default=False)', '--exact' , 'store_true', bool),
		('start_year', 0, False, 'Year to start search from(min: 1800 onwards)', '-sy', 'store', int),
		('end_year', 0, False, 'Year till which to search for(max: present year)', '-ey', 'store', int),
		('lang', '', False, 'Language in which to search(i.e. English)', '-ln', 'store', str),
		('ext', '', False, 'Extension of the ebook or article(i.e. pdf, epub, txt, rar, mobi)', '-e', 'store', str),
		('count', 50, False, 'No. of results to show(default=50)', '-c', 'store', int) # 50 entries per page
	),
    'examples': ('zlibrary -q <QUERY> -e pdf',)
}

def table_format(data):
	from bs4 import BeautifulSoup

 	# Declaring output_table here in case no table found on page, return this one
	output_table = []
	raw_data = BeautifulSoup(data, 'lxml')
	for table in raw_data.find_all('table', {'class': 'table_book'}):	
		# If this statement is not used, table will reset each time and only 50 entries will be there
		if output_table == []:
			output_table = [ {header.text:[]} for header in table.findAll('th', {'class': 'header'}) ]
		for row in table.find_all('tr'):
			# Using zip to avoid cartesian product caused by 2 loops
			for row_data, column in zip(row.find_all('td'), output_table):
				if row_data.find('div',{'class':'authors'}):
					# Removing integer in front of author name, regex = remove digits, tabs, newline
					row_data = re.sub(r"[0-9\n\t]", '', row_data.text)
				else:
					# Using split to clean other columns except author
					row_data = ' '.join(row_data.text.split())
				# Find the key of present column
				key = [ i for i in column.keys() ][0]
				# Adding entire row data in one dump to a column
				column[key].append(row_data) 
	return output_table

def module_api(self):
	query = self.options['query']
	exact = self.options['exact']
	start_year = self.options['start_year']
	end_year = self.options['end_year']
	lang = self.options['lang']
	extension = self.options['ext']
	count = self.options['count'] 
	output = {'results': [ {'books':[]}, {'articles':[]} ] }

	run = self.zlibrary(query, exact, start_year, end_year, lang, extension, count)
	if run.search() == 'False':
		return output
	
	books_table = table_format(run.books_data)
	articles_table = table_format(run.articles_data)
	
	# Even if output contains more results, save only as much as count
	output['results'][0]['books']	= [ {list(i.keys())[0]: list(i.values())[0][:count] } for i in books_table] 
	output['results'][1]['articles'] = [ {list(i.keys())[0]: list(i.values())[0][:count] } for i in articles_table]
	
	self.save_gather(output, 'search/zlibrary', query, output=self.options.get('output'))
	return output

def module_run(self):
	output = module_api(self)

	for src in ['books', 'articles']:
		pos = ['books', 'articles'].index(src) # Index to see books data or articles data
		base = output['results'][pos][src] # The base to access items
		
		no_of_columns = len(base)
		no_of_rows =  0 if len(base) == 0 else \
		len([ i for i in base[0].values() ][0])
		# Populating table headers
		columns = [j for i in base for j in i.keys() ] 

		for row_index in range(0, no_of_rows): 
			data = []
			for col_index in range(0, no_of_columns):
				key_name = columns[col_index]
				data.append([ columns[col_index], base[col_index][key_name][row_index] ])
			self.table(data, ['Key','Value'], title=f"{src}: Entry No. {row_index+1}") 




"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=10):
		""" core.ac.uk search engine

				q         : query for search
				limit     : maximum result count
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._json = []
		self._results = []
		self.url = 'https://core.ac.uk/search/api/search'

	def run_crawl(self):
		self.framework.verbose('Searching the CORE domain...')

		data = f'{{"basicQuery":{{"count":"{self.limit}","searchCriteria":"{self.q}","offset":0}}}}'
		try:
			req = self.framework.request(
					url=self.url,
					method='POST',
					data=data,
					)
		except Exception as e:
			self.framework.error(f"ConnectionError {e}.", 'util/search/core_ac', 'run_crawl')
			self.framework.error('CORE.AC.UK is missed!', 'util/search/core_ac', 'run_crawl')
			return
		else:
			self._json = req.json()

	@property
	def json(self):
		return self._json

	@property
	def results(self):
		findlink = lambda x: x["downloadUrl"]
		findauthors = lambda x: x.get("authorsString")
		findtitle = lambda x: x["title"]
		findsummary = lambda x: x.get("snippet")

		for count,article in enumerate(self._json['results']):
			if count == self.limit:
				break
			d = findsummary(article)
			c = findauthors(article)
			self._results.append({
				't': findtitle(article),
				'a': findlink(article),
				'c': c if c is not None else 'Authors not available',
				'd': re.sub('<[^<]+?>', '', d) if d is not None\
						else 'Abstract not available'
				})

		return self._results



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from json import dumps

meta = {
	'name': 'Reddit Search',
	'author': 'Kaushik',
	'version': '0.1',
	'description': 'Search tweets from twitter',
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
		('count', 15, False, 'Max result count (default=15)', '-l', 'store', int),
		('sortby', 'relevance', False, 'Sort resuts by (relevance|hot|top|new|comments)', '-s', 'store', str),
		('verbose', False, False, 'Print all post details as json', '-v', 'store_true', bool),
	),
	'examples': ('reddit_search -q <QUERY> -l 15',
		'reddit_search -q <QUERY> -l 15')
}


def module_api(self):
	query = self.options['query']
	count = self.options['count']
	sortby = self.options['sortby']
	verbose = self.options['verbose']

	run = self.reddit(query, count, sortby, verbose)
	run.run_crawl()

	output = {'results': run.results}

	self.save_gather(output, 'osint/twitter', query, output=self.options['output'])
	return output

	

def module_run(self):
	output = module_api(self)
	self.search_engine_results(output)



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

meta = {
	'name': 'Pwned database search',
	'author': 'Vikas Kundu',
	'version': '1.1',
	'description': 'Search your email for data breaches',
	'comments': (
			'Using XmlHttp API of haveibeenpwned.com',
			'to get JSON data'
	),
	'sources': ('haveibeenpwned.com',),
	'options': (
		('email', None, True, 'Email to search for breach', '-e', 'store', str),
	),
	'examples': ('email_pwned -e <email> --output',)
}


def scrap(email):
	import cloudscraper
	url = f"https://haveibeenpwned.com/unifiedsearch/{email}"
	scraper = cloudscraper.create_scraper()
	result = scraper.get(url)
	if result.text:
		return result.json()
	else:
		return False


def module_api(self):
	output = {'breaches':[], 'pastes':[]}
	email = self.options['email']
	self.verbose('[PAWNED] Searching for pwning...')
	pwns = scrap(email)
	if pwns:
		output['breaches'] = [{'breach_name': x['Name'], 'breach_domain': x['Domain']} for x in pwns['Breaches']]
		if pwns['Pastes']:
			output['pastes'] = [{'paste_id': x['Id'], 'paste_source': x['Source']} for x in pwns['Pastes']]

	self.save_gather(output, 'osint/email_pwned', email,
					 output=self.options['output'])
	return output


def module_run(self):
	output = module_api(self)

	for section in output.keys():
		if isinstance(output[section], str):
			self.output(output[section])
			break
		rows = []
		for data in output[section]:
			rows.append(list(data.values()))
		self.table(rows, ['breaches', 'pastes'], section)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:
	# framework = None
	def __init__(self, q, limit=1):
		""" dogpile.com search engine

			q     : Query for search
			limit : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.url = 'https://www.dogpile.com/serp'
		self.next_page_text = 'pagination__num pagination__num--next-prev pagination__num--next'
		self._pages = ''
		self.limit = limit
		self.xpath_name = {
			'results': '//div[@class="web-bing__result"]',
			'results_content': './/span[@class="web-bing__description"]',
			'results_title_a': './/a[@class="web-bing__title"]',
		}
		self.xpath = {
			self.xpath_name['results']: [
				self.xpath_name['results_content'],
				self.xpath_name['results_title_a'],
			]
		}

	def run_crawl(self):
		page = 1
		payload = {'page': page, 'q': self.q, 'sc': ''}
		tries = 0
		while True:
			self.framework.verbose(f"[Dogpile] Searching in {page} page...", end='\r')
			try:
				req = self.framework.request(
					url=self.url,
					params=payload,
					allow_redirects=True)
			except Exception as e:
				self.framework.error(f"ConnectionError: {e}", 'util/dogpile', 'run_crawl')
				tries += 1
				if tries >= 2:
					break
			else:
				if req.status_code == 307:
					self.framework.error('Dogpile CAPTCHA triggered.', 'util/dogpile', 'run_crawl')
					break
				self._pages += req.text
				if page >= self.limit or self.next_page_text not in req.text:
					break
				page += 1
				payload['page'] = page
				sc = re.search(fr'page={page}&amp;sc=([\w]+)"', req.text)
				if not sc:
					break
				payload['sc'] = sc.group(1)

	@property
	def results(self):
		parser = self.framework.page_parse(self._pages)
		xpath_results = parser.html_fromstring(self.xpath)
		results = []
		if not xpath_results:
			return results
		root = xpath_results[self.xpath_name['results']]
		for i in range(len(root[self.xpath_name['results_title_a']])):
			a = root[self.xpath_name['results_title_a']][i].get('href')
			cite = self.framework.meta_search_util().make_cite(a)
			result = {
				't': root[self.xpath_name['results_title_a']][i].text_content(),
				'a': a,
				'c': cite,
				'd': root[self.xpath_name['results_content']][i].text_content(),
			}
			results.append(result)
		return results

	@property
	def pages(self):
		return self._pages

	@property
	def links(self):
		links = [x['a'] for x in self.results]
		return links

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Onions Network Search',
	'author': 'Saeed',
	'version': '0.4',
	'description': 'onion_search is used to create the premier \
	search engine for services residing on the Tor anonymity network.',
	'sources': ('ahmia', 'onionland', 'darksearch'),
	'options': (
		('query', None, True, 'Domain Name,\
			Company Name, keyword, etc', '-q', 'store', str),
	),
	'examples': ('onion_search -q <KEYWORD|COMPANY>', 'onion_search -q <KEYWORD|COMPANY> --output')
}

def module_api(self):
	q = self.options['query']
	ahmia = self.ahmia(q)
	ahmia.run_crawl()
	links = ahmia.links
	output = {'links': []}
	onionland = self.onionland(q, limit=5)
	onionland.run_crawl()
	links.extend(onionland.links)

	darksearch = self.darksearch(q, limit=1)
	darksearch.run_crawl()
	links.extend(darksearch.links)

	output['links'] = list(set(links))
	self.save_gather(output, 'osint/onion_search', q, output=self.options['output'])
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from re import search

class main:

	def __init__(self, content, headers):
		""" Detect web language

			content	  : web content
			headers	  : web headers
		"""
		self.content = content
		self.headers = headers
		self._lang = None

	def run_crawl(self):
		for attr in dir(self):
			con1 = not attr.startswith('__')
			con2 = not attr.endswith('__')
			con3 = attr not in ('_lang', 'content', 'headers',
							 'lang', 'run_crawl', 'framework')
			if con1 and con2 and con3:
				getattr(self, attr)()
		
	def ruby(self):
		if search(r'<a href\=\S*(\.rb|\.rhtml)', self.content):
			self._lang = 'Ruby'

	def asp(self):
		if search(r'<a href\=\S*(\.asp)', self.content):
			self._lang = 'ASP'

	def asp_net(self):
		if search(r'<a href\=\S*(\.aspx|\.axd|\.asx|\.asmx|\.ashx|\.asax|\.ascx|\.cshtml)', self.content):
			self._lang = 'ASP.NET'

	def cold_fusion(self):
		if search(r'<a href\=\S*(\.cfm|\.cfml)', self.content):
			self._lang = 'ColdFusion'

	def flash(self):
		if search(r'<a href\=\S*(\.swf)', self.content):
			self._lang = 'Flash'

	def perl(self):
		if search(r'<a href\=\S*(\.pl|\.cgi)', self.content):
			self._lang = 'Perl'

	def python(self):
		if search(r'<a href\=\S*(\.py)', self.content):
			self._lang = 'Python'

	def php(self):
		if search(r'<a href\=\S*(\.php|\.php2|\.php3|\.php4|\.php5|\.phtm|\.phtml)', self.content):
			self._lang = 'PHP'

	def java(self):
		if search(r'<a href\=\S*(\.do|\.jhtml|\.jsp|\.jspa|\.jspx|\.jws|\.wss|\.action)', self.content):
			self._lang = 'Java'

	@property
	def lang(self):
		return self._lang



"""
OWASP Maryam!
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

meta = {
	'name': 'Openstreetmap',
	'author': 'Kunal Khandelwal',
	'version': '0.1',
	'description': "OpenStreetMap is the free wiki world map",
	'sources': ('openstreetmap',),
	'options': (
		('query', None, True, 'Query string', '-q', 'store', str),
	),
	'examples': ('openstreetmap -q <QUERY>',)
}


def module_api(self):
	query = self.options['query']
	run = self.openstreetmap(query)
	run.run_crawl()
	results = run.results
	output = {'results': results}
	self.save_gather(output, 'search/openstreetmap', query, output=self.options['output'])
	return output

def module_run(self):
	output = module_api(self)['results']
	for item in output:
		print()
		if item['category'] != '':
			self.output(item['category'])
		self.output(item['location'])
		coordinates = f"{item['latitude']}, {item['longitude']}"
		self.output(coordinates)
		self.output(item['link'])



#!/usr/bin/env python3
"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from setuptools import setup, find_packages

requirements_file = open('requirements', 'r')
requirements = requirements_file.readlines()
for i, line in enumerate(requirements):
	requirements[i] = line.strip()

setup(
	name='maryam',
	version='2.5.1-0',
	url='https://github.com/saeeddhqan/Maryam',
	author='Saeed Dehqan',
	author_email='saeed.dehghan@owasp.org',
	packages=find_packages(),
	include_package_data=True,
	package_data={"maryam": ['data/*']},
	license='GPL-V3',
	description='OWASP Maryam is a modular/optional open-source framework based on OSINT and data gathering.',
	long_description=open('README.md').read(),
	long_description_content_type='text/markdown',
	keywords=['OWASP', 'OSINT', 'search-engine', 'social-networks', 'Maryam'],
	scripts=['bin/maryam'],
	install_requires=requirements,
	classifiers=[
		'Programming Language :: Python :: 3.10',
		'Development Status :: 5 - Production/Stable',
		'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
		'Natural Language :: English',
		'Operating System :: POSIX :: Linux',
		'Environment :: Console',
	]
)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from re import search, I

class main:

	def __init__(self, content, headers):
		""" Detect operation system

			content 	: Web page content
			headers		: Web page headers
		"""
		self.content = content
		self.headers = headers
		self._os = None

	def run_crawl(self):
		for attr in dir(self):
			con1 = not attr.startswith('__')
			con2 = not attr.endswith('__')
			con3 = attr not in ('_os', 'content', 'headers',
							 'framework', 'os', 'run_crawl')
			if con1 and con2 and con3:
				getattr(self, attr)()
		
	def windows(self):
		OSes = ('windows', 'win32')
		for os in OSes:
			for header in self.headers.items():
				if search(os, header[1], I):
					self._os = os.title()

	def bsd(self):
		for header in self.headers.items():
			if search(r'^bsd', header[1], I):
				self._os = 'BSD'

	def ibm(self):
		OSes = ('IBM', 'Lotus-Domino', 'WebSEAL')
		for os in OSes:
			for header in self.headers.items():
				if search(os, header[1], I):
					self._os = 'IBM'

	def linux(self):
		OSes = ('linux', 'ubuntu', 'gentoo', 'debian', 'dotdeb', 'centos', 'redhat', 'sarge', 'etch',
			  'lenny', 'squeeze', 'wheezy', 'jessie', 'red hat', 'scientific linux')
		for os in OSes:
			for header in self.headers.items():
				if search(os, header[1], I):
					self._os = os.title()

	def mac(self):
		for header in self.headers.items():
			if search(r'^mac|^macos', header[1], I):
				self._os = 'MacOS'

	def solaris(self):
		OSes = ('solaris', 'sunos', 'opensolaris', 'sparc64', 'sparc')
		for os in OSes:
			for header in self.headers.items():
				if search(os, header[1], I):
					self._os = os.title()

	def unix(self):
		for header in self.headers.items():
			if search(r'^unix', header[1], I):
				self._os = 'Unix'

	@property
	def os(self):
		return self._os



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

meta = {
	'name': 'Discord Search',
	'author': 'Divya Goswami',
	'version': '1.0',
	'description': 'Search the name of any user or server on discord [from disboard and discordhub]',
	'sources': ('discord',),
	'options': (
		('user', None, False, 'User flag (search for a user based on the integer hash)', '-u', 'store', str),
		('server', None, False, 'Server flag (search for a server)', '-s', 'store', str),
		('limit', 3, False, 'Search limit(number of pages, default=3)', '-l', 'store', int),
		('engine', 'discord', False, 'Engine used is discord', '-e', 'store', str),
		('thread', 2, False, 'The number of engine that run per round(default=2)', '-t', 'store', int),
	),
	'examples': ('discord -u <USER HASH> --output', 'discord -s <SERVER-NAME> -l 5')
	}

PAGES = ''

def search(self, name, q, q_formats, limit, count):
	global PAGES, LINKS
	engine = getattr(self, name)
	varnames = engine.__init__.__code__.co_varnames
	if 'limit' in varnames:
		attr = engine(q, limit)
	attr.run_crawl()
	PAGES += attr.pages

def module_api(self):
	global PAGES
	user = self.options['user']
	server = self.options['server']
	limit = self.options['limit']
	engine = ['discord']
	output = { 'servers': [], 'users': [] }
	if user and server:
		query = f"{user}_u"
	elif user:
		query = f"{user}_u"
	elif server:
		query = f"{server}_s"
	else:
		return output
	u_list = []
	s_list = []
	self.thread(search, self.options['thread'], engine, query, {}, limit, 0, meta['sources'])
	user_reg = r'<h4 class="title"><a href="\/profile\/(\d+)">(.+)<\/a'
	server_reg = r'<img src="https:\/\/cdn\.discordapp\.com\/icons\/(\d+).+alt="(.+)" width'
	u_list += [i for i in set(re.findall(user_reg, PAGES))]
	s_list = [i for i in set(re.findall(server_reg, PAGES))]
	output['servers'] += [f"{i[1]}\n Join: \thttps://disboard.org/server/join/{i[0]}" for i in s_list]
	output['users'] += [f"{i[1]}\n Check Full Profile: \thttps://discord.id/?prefill={i[0]}" for i in u_list]

	self.save_gather(output, 'search/discord', query, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re
import webbrowser

class main:

	def __init__(self, q):
		""" carrot2.org search engine

			q 		  : Query for search
		"""
		self.framework = main.framework
		self.q = q
		self._pages = ''
		self._json = {}
		self.etools = 'https://www.etools.ch/partnerSearch.do'

	def run_crawl(self):
		params = {'country': 'web', 'dataSourceResults': 40, 
				'dataSources': 'all', 'language': 'en', 'maxRecords': 200, 'partner': 'Carrot2Json', 
				'query': self.q, 'safeSearch': 'false'}
		headers = {'Host': 'www.etools.ch', 
				   'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:74.0) Gecko/20100101 Firefox/74.0',
				   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 
				   'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate, br', 
				   'Connection': 'keep-alive', 'Cookie': 'JSESSIONID=6120E7C52197190DE5126DCBF47D38B0', 
				   'Upgrade-Insecure-Requests': '1', 'Cache-Control': 'max-age=0'}
		self.framework.debug(f"[eTOOLS] Searching in 'etools.ch'...")
		try:
			req = self.framework.request(url=self.etools, params=params, headers=headers, allow_redirects=True)
		except:
			self.framework.error('ConnectionError.', 'util/carrot2', 'run_crawl')
			self.framework.error('eTOOLS is missed!', 'util/carrot2', 'run_crawl')
		else:
			self._pages = req.text
			try:
				self._json = req.json()
			except:
				self.framework.error('eTOOLS is missed!', 'util/carrot2', 'run_crawl')

	@property
	def pages(self):
		return self._pages

	@property
	def results(self):
		resp = self._json.get('response')
		results = []
		if resp:
			merged_records = resp.get('mergedRecords')
			for result in merged_records:
				results.append({
					't': result['title'], 
					'a': result['url'], 
					'c': self.framework.meta_search_util().make_cite(result['url']),
					'd': result['text']})
		return results

	@property
	def links(self):
		self._links = []
		resp = self._json.get('response')
		if resp:
			resp = resp.get('mergedRecords')
			for link in resp:
				self._links.append(link.get('url'))
		return self._links

	@property
	def json(self):
		return self._json

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class main:

	def __init__(self, q, limit=2):
		""" millionshort.com search engine

			q 		  : query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = q
		self.limit = limit
		self._pages = ''
		self._json = []
		self._links = []
		self._links_with_title = {}
		self.millionshort = 'https://millionshort.com/api/search'

	def run_crawl(self):
		page = 1
		set_page = lambda x: (x - 1) * 10 + 1
		payload = {'keywords': self.q, 'remove': '0', 'offset': '0'}
		headers = {
			"Host": 'millionshort.com',
			"User-Agent": 'Mozilla/5.0 (X11; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0',
			"Accept": 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
			"Accept-Language": 'en-US,en;q=0.5',
			"Accept-Encoding": 'gzip, deflate, br',
			"Connection": 'keep-alive',
			"Cookie": 'connect.sid=s%3AVZd6oWjMPfegJTrCK6iVYs8p1S4q3Q-h.OC7iYFkDE5A2u6%2BsJkCa96q6ituwLnNDh0EjzEJhvNk',
			"Upgrade-Insecure-Requests": '1',
			"If-None-Match": 'W/"9fc-Kb1JjxDz/K4kA1S7nkbpkibKV94"',
			"Cache-Control": 'max-age=0'
		}
		while True:
			self.framework.verbose(f"[MILLIONSHORT] Searching in {page} page...")
			try:
				req = self.framework.request(url=self.millionshort, params=payload, headers=headers)
			except Exception as e:
				self.framework.error(f"ConnectionError {e}.", 'util/engines/millionshort', 'name_crawl')
				self.framework.error('millionshort is missed!', 'util/engines/millionshort', 'name_crawl')
				break
			else:
				if 'captcha' in req.json():
					self.framework.error('CaptchaError', 'util/engines/millionshort', 'run_crawl')
					self.framework.error('millionshort is missed!', 'util/engines/millionshort', 'name_crawl')
					return
				self._json.append(req.json())
				self._pages += req.text
				if page == self.limit:
					break
				page += 1
				payload['offset'] = set_page(page)

	@property
	def pages(self):
		return self._pages

	@property
	def json(self):
		return self._json

	@property
	def links(self):
		for page in self._json:
			results = page.get('content', {}).get('webPages', {})
			self._links.extend([x.get('displayUrl') for x in results])
		return self._links

	@property
	def results(self):
		results = []
		for page in self._json:
			items = page.get('content', {}).get('webPages', {})
			for item in items:
				a = item['displayUrl']
				result = {
					't': item['name'],
					'a': a,
					'c': self.framework.meta_search_util().make_cite(a),
					'd': item['snippet'],
				}
				results.append(result)
		return results

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q, self.links)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q, self.links)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import urllib
import json
import datetime

from maryam.core.basedir import BASEDIR

meta = {
	'name': 'Bing Mobile View',
	'author': 'Vikas Kundu',
	'version': '0.1',
	'description': 'Get screenshot of a URL in Bing mobile friendly view',
	'sources': ('bing',),
	'options': (
		('url', None, True, 'URL whose mobile screenshot needs to be taken', '-u', 'store', str),
		('blacklist', None, False, 'If this image is found retry, available='\
		'[linkedin_authwall_en, add-your-custom]', '-b', 'store', str),
		('retries', 2, False, 'Number of times to try again if blacklist image found, default=2', '-r', 'store', int),
	),
	'examples': ('bing_mobile_view -u <URL>',)
}

def blacklist_check(self, image_data):
	black_img_path = os.path.join(BASEDIR, 'data', 'images', 'blacklist', f'{self.options["blacklist"]}.json')
	if not os.path.isfile(black_img_path) : # Check if blacklisted image exists?
		self.error('The blacklisted image name entered by user does not exist.', 'bing_mobile_view', 'blacklist_check')
		return False

	blacklisted_image_data = json.load(open(black_img_path)) # Load the data of blacklisted image
	counter, run = 1, self.bing_mobile_view(self.options['url'])
	while blacklisted_image_data['img-data'] == image_data and counter <= self.options['retries']:
		self.verbose(f"[Bing Mobile View] Image scraped is blacklisted. Retry No:{counter}")
		if run.screenshot() == False:
			self.verbose(f"[Bing Mobile View] Error on retry no. {counter}. Maybe this is not a public URL.")
			return False
	
		image_data = run.raw_image_data
		counter += 1
	
	if counter-1 == self.options['retries']: # If all retries are done then return false otherwise image data
		self.error('Exhausted retries, maybe this is not a public profile.', 'bing_mobile_view', 'blacklist_check')
		return False
	else:
		return image_data

def module_api(self):
	url = self.options['url']
	blacklisted_img_name = self.options['blacklist']
	retries = self.options['retries']
	output = {'Image-Location': []}

	run = self.bing_mobile_view(url)

	if run.screenshot() == 'False': # If some error has occured while taking screenshot, return output to avoid further errors.
		return output

	image_data = run.raw_image_data

	if image_data == '':
		self.error('Unable to detect image in the page', 'bing_mobile_view', 'module_api')
		return output

	'''
	Blacklisted images are images like authwall of linkedin, if they are found, retry.
	To add your own image, go to data/images/blacklist/ and save in file_name.json format
	like {'img-data': 'base-64-data-of-image'}. Then pass it using -b param i.e. -b custom-blacklist-image
	For more info, see add-your-custom.json file.
	'''
	if blacklisted_img_name is not None:
		self.verbose('[Bing Mobile View] Checking if the image obtained is blacklisted...')
		image_data = blacklist_check(self, image_data)	
		if not image_data: # If some error while blacklist_check function then exit
			return output
			
	'''
	When the image file is finally saved, it is saved inside the customer folder which is the same as
	the domain name. The rest of the url is the file name. i.e. if url = https://www.example.com/abc
	then file will be saved inside www.example.com folder and its name will be abc-timestamp.jpg
	where timestamp is added to make each image name unique.	
	'''

	folder_name = urllib.parse.urlparse(str(url)).netloc if urllib.parse.urlparse(str(url)).netloc \
	else urllib.parse.quote_plus(url) #Folder name is the domain name. If empty then folder name == URL
	file_name = f"{urllib.parse.quote_plus(url)} {str(datetime.datetime.now())}.jpg" #File name is url encode+timestamp
	
	folder_filepath = os.path.join(BASEDIR, 'data', 'images', folder_name)

	if not os.path.isdir(folder_filepath) : # If no pre-existing folder for image, make one
		os.mkdir(folder_filepath)
	
	final_filepath = os.path.join(folder_filepath, f'{file_name}')

	self.verbose('[Bing Mobile View] Dumping the image...')	
	response = urllib.request.urlopen(image_data)
	with open(final_filepath, 'wb') as f:
        	f.write(response.file.read())
    
	output['Image-Location'] += [str(final_filepath)]
	
	self.save_gather(output, 'osint/bing_mobile_view', url, output=self.options.get('output'))
	return output

def module_run(self):
	self.alert_results(module_api(self))



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from re import search

class main:

	def __init__(self, q, limit=2):
		""" baidu.com search engine
			
			q 		  : Query for search
			limit	  : Number of pages
		"""
		self.framework = main.framework
		self.q = self.framework.urlib(q).quote
		self.limit = 25 if limit > 25 else limit
		self._pages = ''
		self.baidu = 'baidu.com'

	def run_crawl(self):
		set_page = lambda x: (x - 1) * 10 + 1
		urls = [f"https://{self.baidu}/s?wd={self.q}&oq={self.q}&pn={set_page(i)}&ie=utf-8" for i in range(1, self.limit+1)]
		max_attempt = len(urls)
		for url in range(len(urls)):
			self.framework.verbose(f"[BAIDU] Searching in {url} page...")
			try:
				req = self.framework.request(url=urls[url])
			except:
				self.framework.error('ConnectionError', 'util/baidu', 'run_crawl')
				max_attempt -= 1
				if max_attempt == 0:
					self.framework.error('Baidu is missed!', 'util/baidu', 'run_crawl')
					break
			else:
				self._pages += req.text

	@property
	def pages(self):
		return self._pages

	@property
	def dns(self):
		return self.framework.page_parse(self._pages).get_dns(self.q)

	@property
	def emails(self):
		return self.framework.page_parse(self._pages).get_emails(self.q)

	@property
	def docs(self):
		return self.framework.page_parse(self._pages).get_docs(self.q)



"""
OWASP Maryam!

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class main:

	def __init__(self, title, author, limit, order):
		""" Search in the Worldcat API: http://classify.oclc.org/classify2/Classify
		
			title	: title of the book
			author	: author of the book
			limit	: maximum queries to show
			order	: order in which to show queries ascending or descending
		"""
		self.framework = main.framework
		self.title = title
		self.author = author
		self.limit = limit
		self.order = f"thold {order}"
		self._xml_data = ''
				
	def run_crawl(self):
		self.framework.verbose('[Worldcat Search] Fetching data for the query...')
		api_url = 'http://classify.oclc.org/classify2/Classify'
		url_with_payload = f'{api_url}?title={self.title}&author={self.author}&maxRecs={self.limit}&orderBy={self.order}'
		try:
			response = self.framework.request(url=url_with_payload)
		except Exception as e:
			self.framework.error('Worldcat is missed!', 'util/worldcat', 'search')
			return False
		else:
			self._xml_data = response.content

	@property
	def xml_data(self):
		return self._xml_data
